/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/



#include <QDebug>
#include <QDir>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QSqlRecord>
#include <QStandardPaths>
#include <QDebug>
#include <exception>
#include <QSqlError>
#include <QIODevice>
#include <QTime>
#include <QDateTime>
#include <QMimeType>

#include "persistancelayer.h"

PersistanceLayer *PersistanceLayer::persistanceLayer = nullptr;

PersistanceLayer *PersistanceLayer::instance()
{
    if ( persistanceLayer == nullptr ) {
        persistanceLayer = new PersistanceLayer;
        persistanceLayer->init();
    }

    return persistanceLayer;
}

void PersistanceLayer::initShema()
{
    QSqlQuery pragmaquery;
    pragmaquery.prepare( QStringLiteral( "PRAGMA synchronous = OFF" ) );

    if ( !pragmaquery.exec() ) {
        qDebug() << pragmaquery.lastError();
    }

    //    pragmaquery.prepare("PRAGMA locking_mode = EXCLUSIVE");
    //    if(!pragmaquery.exec()){
    //        qDebug() << pragmaquery.lastError();
    //    }
    pragmaquery.prepare( QStringLiteral( "PRAGMA JOURNAL_MODE=MEMORY" ) );

    if ( !pragmaquery.exec() ) {
        qDebug() << pragmaquery.lastError();
    }

    pragmaquery.prepare( QStringLiteral( "PRAGMA TEMP_STORE=MEMORY" ) );

    if ( !pragmaquery.exec() ) {
        qDebug() << pragmaquery.lastError();
    }

    transaction();

    QSqlQuery query;
    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS user_data "
                                   "(id integer primary key,"
                                   "username varchar(40),"
                                   "password varchar(40),"
                                   "token varchar(100),"
                                   "token_expire integer,"
                                   "user_id varchar(12),"
                                   "server varchar(100))" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS rooms "
                                   "(id varchar(12) primary key, "
                                   "name varchar(100),"
                                   "type varchar(10),"
                                   "read_only bool,"
                                   "muted text,"
                                   "archived bool,"
                                   "joined bool, "
                                   "blocked bool DEFAULT 0, "
                                   "username varchar(100) DEFAULT \"\")" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS messages "
                                   "(id varchar(12) primary key,"
                                   "rid varchar(12),"
                                   "author varchar(30),"
                                   "user_id varchar(12),"
                                   "ts integer,"
                                   "read BOOLEAN DEFAULT FALSE,"
                                   "json text)" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    query.prepare( QStringLiteral( "CREATE INDEX msgRidIndex "
                                   "ON messages (rid)" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS file_cache "
                                   "(url varchar(200) primary key,"
                                   "path varchar(200),"
                                   "ts integer)" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS current_room"
                                   "(id integer primary key,"
                                   "rid varchar(12),"
                                   "name varchar(30))" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS custom_emojis"
                                   "(id varchar(30) primary key,"
                                   "file varchar(200),"
                                   "html varchar(200),"
                                   "unicode varchar(20),"
                                   "category varchar(100),"
                                   "sort_order integer DEFAULT 1)" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS blocked_users"
                                   "(id varchar(30) primary key, author varchar(30) )" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }

    /* query.prepare( QStringLiteral( "CREATE INDEX authorIdx "
                                    "ON blocked_users (author)" ) );

     if ( !query.exec() ) {
         qWarning() << query.lastError();
     }
     */

    commit();

    query.prepare( QStringLiteral( "SELECT db_ver FROM app_info WHERE id=0" ) );
    int dbVer = -1;

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    } else {
        dbVer = 0;
        QSqlRecord rec = query.record();
        int dbVerIndex = rec.indexOf( QStringLiteral( "db_ver" ) );

        while ( query.next() ) {
            dbVer = query.value( dbVerIndex ).toInt();
        }
    }

    if ( !dbVer ) {
        upgradeSchema();
    } else if ( dbVer == -1 ) {
        transaction();
        query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS app_info"
                                       "(id varchar(30) primary key,"
                                       "db_ver integer,"
                                       "app_ver integer)" ) );

        if ( !query.exec() ) {
            qWarning() << query.lastError();
        }

        commit();
        QString dbVersion = QString::number( DBVERSION );
        query.prepare( QStringLiteral( "INSERT INTO app_info (db_ver) VALUES(" ) + dbVersion + ")" );

        if ( !query.exec() ) {
            qWarning() << query.lastError();
        }
    }

    query.prepare( QStringLiteral( "CREATE TABLE IF NOT EXISTS app_settings"
                                   "(id integer primary key,"
                                   "serverid integer,"
                                   "property varchar(100) UNIQUE,"
                                   "value varchar(100))" ) );

    if ( !query.exec() ) {
        qWarning() << query.lastError();
    }



    query.prepare( QStringLiteral( "SELECT id FROM custom_emojis" ) );

    if ( !query.exec() ) {
        qWarning() << "failed SELECT custom_emojis";
        qWarning() << query.lastError();
    } else {
        int size = 0;

        while ( query.next() ) {
            size++;
            break;
        }

        if ( size <= 0 ) {
            QFile dump( QStringLiteral( ":/sql/emojis.sql" ) );

            if ( !dump.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
                qWarning() << "error loading dump";
                qWarning() << dump.errorString();
            } else {
                transaction();
                QTextStream in( &dump );
                QString dumpString;
                dumpString.reserve( 200 );

                while ( !in.atEnd() ) {

                    dumpString = in.readLine();

                    query.prepare( dumpString );

                    if ( !query.exec() ) {
                        qWarning() << "failed execute import dump query";
                        qWarning() << query.lastError();
                    }
                }

                dump.close();
                commit();
            }
        }
    }
}

void PersistanceLayer::upgradeSchema()
{
    QSqlQuery query;
    query.prepare( QStringLiteral( "SELECT db_ver FROM app_info" ) );
    int dbVersion = 0;

    if ( !query.exec() ) {
        qWarning() << "failed execute app_version query";
    } else {
        QSqlRecord rec = query.record();
        int dbVerIndex = rec.indexOf( "db_ver" );

        while ( query.next() ) {
            dbVersion = query.value( dbVerIndex ).toInt();
        }

        if ( dbVersion < DBVERSION ) {
            QFile sqlFile;
            int currentTarget = 0;

            if ( dbVersion < 1 ) {
                currentTarget = 1;
                sqlFile.setFileName( QStringLiteral( ":/sql/migrations/1.sql" ) );
                query.prepare( QStringLiteral( "DELETE FROM custom_emojis" ) );

                if ( query.exec() ) {
                    qWarning() << "truncation of, custom_emojis failed";
                    qWarning() << query.lastError();
                }
            } else if ( dbVersion < 2 ) {
                currentTarget = 2;
                sqlFile.setFileName( QStringLiteral( ":/sql/migrations/2.sql" ) );
            } else if ( dbVersion < 3 ) {
                currentTarget = 3;
                sqlFile.setFileName( QStringLiteral( ":/sql/migrations/3.sql" ) );
            } else if ( dbVersion < 4 ) {
                currentTarget = 4;
                sqlFile.setFileName( QStringLiteral( ":/sql/migrations/4.sql" ) );
            }

            if ( !sqlFile.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
                qWarning() << "error loading dump";
                qWarning() << sqlFile.errorString();
            } else {
                transaction();

                QTextStream in( &sqlFile );
                QString line;

                while ( !in.atEnd() ) {
                    line = in.readLine();
                    query.prepare( line );

                    if ( !query.exec() ) {
                        qWarning() << "migration to DB ver " << currentTarget << " failed";
                        qWarning() << query.lastError();
                        qWarning() << query.lastQuery();
                        exit( EXIT_FAILURE );
                    }
                }

                commit();
            }

            upgradeSchema();
        }
    }
}

void PersistanceLayer::wipeDb()

{
    mDb.close();
    QString directory = QStandardPaths::writableLocation( QStandardPaths::AppLocalDataLocation );
    QString filePath = directory + QStringLiteral( "/data.sqlite" );
    QFile file( filePath );

    if ( !file.remove() ) {
        qWarning() << file.errorString();
    } else {
        qDebug() << "db wiped";
    }

    init();
}

void PersistanceLayer::transaction( void )
{
    mMutex.lock();

    if ( mCommitCounter == 0 ) {
        if ( !mDb.transaction() ) {
            qWarning() << mDb.lastError();
        }
    }

    mCommitCounter++;
    mMutex.unlock();
}

void PersistanceLayer::commit( void )
{
    mMutex.lock();

    if ( mCommitCounter ) {
        mCommitCounter--;
    }

    if ( mCommitCounter == 0 ) {
        if ( !mDb.commit() ) {
            qWarning() << mDb.lastError();
        }

        counter++;
        qDebug() << "db commits" << counter;
    }

    mMutex.unlock();
}



void PersistanceLayer::setUserName( const QString &pUsername )

{
    transaction();
    QSqlQuery setName;
    setName.prepare( QStringLiteral( "UPDATE user_data "
                                     "SET username=:username WHERE id=:id" ) );
    setName.bindValue( QStringLiteral( ":id" ), 0 );
    setName.bindValue( QStringLiteral( ":username" ), pUsername );

    if ( !setName.exec() ) {
        qWarning() << setName.lastError();
    }

    commit();
}

void PersistanceLayer::setPassword( const QString &pPassword )
{
    transaction();
    QSqlQuery setPass;
    setPass.prepare( QStringLiteral( "UPDATE user_data "
                                     "SET password=:password WHERE id=:id" ) );
    setPass.bindValue( QStringLiteral( ":password" ), pPassword );
    setPass.bindValue( QStringLiteral( ":id" ), 0 );

    if ( !setPass.exec() ) {
        qWarning() << setPass.lastError();
    }

    commit();
}

void PersistanceLayer::setToken( const QString &pToken, uint pExpire )
{
    transaction();
    QSqlQuery setToken;

    setToken.prepare( QStringLiteral( "UPDATE user_data "
                                      "SET token=:token, token_expire=:token_expire WHERE id=:id" ) );
    setToken.bindValue( QStringLiteral( ":token" ), pToken );
    setToken.bindValue( QStringLiteral( ":id" ), 0 );
    setToken.bindValue( QStringLiteral( ":token_expire" ), pExpire );

    if ( !setToken.exec() ) {
        qWarning() << setToken.lastError();
    }

    commit();
}

void PersistanceLayer::setUserData( const QString &pUser, const QString &pPass )
{
    transaction();
    QSqlQuery setUser;

    setUser.prepare( QStringLiteral( "REPLACE INTO user_data "
                                     "(id,password,username) VALUES(:id,:password,:username)" ) );
    setUser.bindValue( ":password", pPass );
    setUser.bindValue( ":id", 0 );
    setUser.bindValue( ":username", pUser );

    if ( !setUser.exec() ) {
        qWarning() << setUser.lastError();
    }

    commit();
}

QString PersistanceLayer::addFileToTemp( const QString &pUrl, const QString &pPath )
{
    QFile file( pPath );
    QString urlPrefix = "";
    QStringList urlSegments = pUrl.split( '/' );
    QString folder;

    if ( urlSegments.length() ) {
        folder = urlSegments.at( urlSegments.length() - 2 );
    } else {
        folder = pUrl;
    }

    QString newFilePath = QStandardPaths::writableLocation( QStandardPaths::TempLocation );

    newFilePath += '/' + folder;
    createFolder( newFilePath );

    if ( !newFilePath.length() ) {
        newFilePath = pPath;
    } else {
        QDir folderPath( newFilePath );

        if ( !folderPath.exists() ) {
            if ( !folderPath.mkpath( newFilePath ) ) {
                qWarning() << "failed creating folder: " << newFilePath;
            }
        }

        newFilePath += '/' + pPath.split( '/' ).last();
        qDebug() << "old path: " << pPath;
        qDebug() << "new path: " << newFilePath;

        if ( pPath != newFilePath ) {
            QFile destFile( newFilePath );

            if ( destFile.exists() ) {
                qDebug() << "file already exists: " << newFilePath << "deleting";

                if ( !destFile.remove() ) {
                    qWarning() << destFile.errorString();
                }
            }

            if ( !file.rename( newFilePath ) ) {
                qWarning() << file.errorString();
            } else {
                //QFile fileOrig( path );

                //                if ( fileOrig.exists() ) {
                //                    if ( !fileOrig.remove() ) {
                //                        qWarning() << fileOrig.errorString();
                //                    }
                //                }
                addFileCacheEntry( pUrl, newFilePath );
                return newFilePath;
            }
        }
    }

    addFileCacheEntry( pUrl, pPath );
    return newFilePath;
}

QString PersistanceLayer::addFileToDocuments( const QString &pUrl, const QString &pPath )
{
    QFile file( pPath );
    QString newFilePath = QStandardPaths::writableLocation( QStandardPaths::DocumentsLocation );

    if ( !newFilePath.length() ) {
        newFilePath = pPath;
    } else {
        QString realname = file.fileName().split( "/" ).last();
        QDir dbPath( newFilePath );

        if ( !dbPath.exists() ) {
            if ( !dbPath.mkpath( newFilePath ) ) {
                qWarning() << "failed to create folder: " << newFilePath;
            }
        }

        newFilePath += +"/" + realname;
        QFile destFile( newFilePath );

        if ( destFile.exists() ) {
            if ( !destFile.remove() ) {
                qWarning() << destFile.errorString();
            }
        }

        if ( !file.rename( newFilePath ) ) {
            qWarning() << file.errorString();
        } else {
            QFile fileOrig( pPath );

            //            if ( fileOrig.exists() ) {
            //                if ( !fileOrig.remove() ) {
            //                    qWarning() << fileOrig.errorString();
            //                }
            //            }
            addFileCacheEntry( pUrl, newFilePath );
            return newFilePath;

        }

    }

    return newFilePath;
}

QString PersistanceLayer::addFileToMusics( const QString &pUrl, const QString &pPath )
{
    QFile file( pPath );
    QString newFilePath = QStandardPaths::writableLocation( QStandardPaths::MusicLocation );

    if ( !newFilePath.length() ) {
        newFilePath = pPath;
    } else {
        QString realname = file.fileName().split( "/" ).last();
        QDir dbPath( newFilePath );

        if ( !dbPath.exists() ) {
            if ( !dbPath.mkdir( newFilePath ) ) {
                qWarning() << "failed to create folder: " << newFilePath;
            }
        }

        newFilePath += +'/' + realname;
        QFile destFile( newFilePath );

        if ( destFile.exists() ) {
            if ( !destFile.remove() ) {
                qWarning() << destFile.errorString();
            }
        }

        if ( !file.rename( newFilePath ) ) {
            qWarning() << file.errorString();
        } else {
            QFile fileOrig( pPath );

            //            if ( fileOrig.exists() ) {
            //                if ( !fileOrig.remove() ) {
            //                    qWarning() << fileOrig.errorString();
            //                }
            //            }
            addFileCacheEntry( pUrl, newFilePath );
            return newFilePath;

        }

    }

    return newFilePath;
}

QString PersistanceLayer::addFileToVideos( const QString &pUrl, const QString &pPath )
{
    QFile file( pPath );
    QString newFilePath = QStandardPaths::writableLocation( QStandardPaths::MoviesLocation );

    if ( !newFilePath.length() ) {
        newFilePath = pPath;
    } else {
        QString realname = file.fileName().split( '/' ).last();
        QDir dbPath( newFilePath );

        if ( !dbPath.exists() ) {
            if ( !dbPath.mkdir( newFilePath ) ) {
                qWarning() << "failed to create folder: " << newFilePath;
            }
        }

        newFilePath += +'/' + realname;

        QFile destFile( newFilePath );

        if ( destFile.exists() ) {
            if ( !destFile.remove() ) {
                qWarning() << destFile.errorString();
            }
        }

        if ( !file.rename( newFilePath ) ) {
            qWarning() << file.errorString();
        } else {
            QFile fileOrig( pPath );

            //            if ( fileOrig.exists() ) {
            //                if ( !fileOrig.remove() ) {
            //                    qWarning() << fileOrig.errorString();
            //                }
            //            }
            addFileCacheEntry( pUrl, newFilePath );
            return newFilePath;

        }

    }

    return newFilePath;
}

QString PersistanceLayer::addFileToImages( const QString &pUrl, const QString &pPath )
{
    QFile file( pPath );
    QString newFilePath = QStandardPaths::writableLocation( QStandardPaths::PicturesLocation );

    if ( !newFilePath.length() ) {
        newFilePath = pPath;
    } else {
        QString realname = file.fileName().split( "/" ).last();
        QDir dbPath( newFilePath );

        if ( !dbPath.exists() ) {
            if ( !dbPath.mkdir( newFilePath ) ) {
                qWarning() << "failed to create folder: " << newFilePath;
            }
        }

        newFilePath += +'/' + realname;
        QFile destFile( newFilePath );

        if ( destFile.exists() ) {
            if ( !destFile.remove() ) {
                qWarning() << destFile.errorString();
            }
        }

        if ( !file.rename( newFilePath ) ) {
            qWarning() << file.errorString();
        } else {
            QFile fileOrig( pPath );

            //                if ( fileOrig.exists() ) {
            //                    if ( !fileOrig.remove() ) {
            //                        qWarning() << fileOrig.errorString();
            //                    }
            //                }
            addFileCacheEntry( pUrl, newFilePath );
            return newFilePath;

        }

    }

    return newFilePath;
}

void PersistanceLayer::createFolder( const QString &pPath )
{
    QDir temp;

    if ( !temp.mkpath( pPath ) ) {
        qWarning() << "failed to create folder: " << pPath;
    }
}

void PersistanceLayer::setUserId( const QString &pUserId )
{
    transaction();
    QSqlQuery setUserId;
    setUserId.prepare( QStringLiteral( "UPDATE user_data SET user_id=:userId WHERE id=:id" ) );
    setUserId.bindValue( QStringLiteral( ":id" ), 0 );
    setUserId.bindValue( QStringLiteral( ":userId" ), pUserId );

    if ( !setUserId.exec() ) {
        qWarning() << setUserId.lastError();
    }

    commit();
}

void PersistanceLayer::setCurrentChannel( const QString &pChannelId, const QString &pChannelName )
{
    transaction();
    QSqlQuery setChannel;
    setChannel.prepare( QStringLiteral( "REPLACE INTO current_room"
                                        "(id,rid,name) VALUES(:id,:rid,:name)" ) );
    setChannel.bindValue( QStringLiteral( ":id" ), 0 );
    setChannel.bindValue( QStringLiteral( ":rid" ), pChannelId );
    setChannel.bindValue( QStringLiteral( ":name" ), pChannelName );

    if ( !setChannel.exec() ) {
        qWarning() << setChannel.lastError();
    }

    commit();
}

void PersistanceLayer::setSetting( const QString &pProperty, const QString &pValue )
{
    transaction();
    QSqlQuery setChannel;
    setChannel.prepare( QStringLiteral( "REPLACE INTO app_settings"
                                        "(property,value) VALUES(:property,:value)" ) );
    setChannel.bindValue( QStringLiteral( ":property" ), pProperty );
    setChannel.bindValue( QStringLiteral( ":value" ), pValue );

    if ( !setChannel.exec() ) {
        qWarning() << setChannel.lastError();
    }

    commit();

}

void PersistanceLayer::addChannel( const QString &pId, const QString &pName, const QString &pType, bool pJoined, bool pReadOnly, const QString &pMuted, bool pArchived, bool pBlocked, const QString &pUsername )
{
    transaction();
    QSqlQuery setChannel;
    setChannel.prepare( QStringLiteral( "REPLACE INTO rooms "
                                        "(id,name,type,joined,read_only,muted,archived,blocked,username) VALUES(:id,:name,:type,:joined,:read_only,:muted,:archived,:blocked,:username)" ) );
    setChannel.bindValue( QStringLiteral( ":id" ), pId );
    setChannel.bindValue( QStringLiteral( ":name" ), pName );
    setChannel.bindValue( QStringLiteral( ":joined" ), pJoined );
    setChannel.bindValue( QStringLiteral( ":type" ), pType );
    setChannel.bindValue( QStringLiteral( ":read_only" ), pReadOnly );
    setChannel.bindValue( QStringLiteral( ":muted" ), pMuted );
    setChannel.bindValue( QStringLiteral( ":archived" ), pArchived );
    setChannel.bindValue( QStringLiteral( ":blocked" ), pBlocked );
    setChannel.bindValue( QStringLiteral( ":username" ), pUsername );

    if ( !setChannel.exec() ) {
        qWarning() << setChannel.lastError();
    }

    commit();
}

void PersistanceLayer::addChannel( const QString &pId, const QString &pName, const QString &pType, bool pJoined, const QString &pUsername )
{
    bool readOnly = false;
    QString muted = "";
    bool archived = false;
    bool blocked = false;
    addChannel( pId, pName, pType, pJoined, readOnly, muted, archived, blocked, pUsername );
}

void PersistanceLayer::deleteChannel( const QString &pId )
{
    QSqlQuery deleteQuery;
    deleteQuery.prepare( QStringLiteral( "DELETE FROM rooms WHERE id=:id" ) );
    deleteQuery.bindValue( QStringLiteral( ":id" ), pId );

    if ( !deleteQuery.exec() ) {
        qWarning() << deleteQuery.lastError();
    } else {
        deleteQuery.prepare( QStringLiteral( "DELETE FROM messages WHERE rid=:id" ) );
        deleteQuery.bindValue( QStringLiteral( ":id" ), pId );

        if ( !deleteQuery.exec() ) {
            qWarning() << deleteQuery.lastError();
        }
    }
}

void PersistanceLayer::addMessage( const QString &pId, const QString &pRid, const QString &pAuthor, qint64 pTs, const QString &pJson, const QString &pUserId )
{
    transaction();
    QSqlQuery addMessage( mDb );
    addMessage.prepare( QStringLiteral( "REPLACE INTO messages "
                                        "(id,rid,author,ts,json,read, user_id) VALUES (:id,:rid,:author,:ts,:json,0,:user_id)" ) );
    addMessage.bindValue( QStringLiteral( ":id" ), pId );
    addMessage.bindValue( QStringLiteral( ":rid" ), pRid );
    addMessage.bindValue( QStringLiteral( ":author" ), pAuthor );
    addMessage.bindValue( QStringLiteral( ":ts" ), pTs );
    addMessage.bindValue( QStringLiteral( ":json" ), pJson );
    addMessage.bindValue( QStringLiteral( ":user_id" ), pUserId );

    if ( !addMessage.exec() ) {
        qWarning() << "id: " << pId << " rid: " << pRid << " author: " << pAuthor << " ts: " << pTs << " json: " << pJson << "userId: " << pUserId;
        qWarning() << addMessage.lastQuery();
        qWarning() << addMessage.executedQuery();
        qWarning() << addMessage.lastError();
    }

    commit();
}

void PersistanceLayer::addFileCacheEntry( const QString &pUrl, const QString &pPath )
{
    transaction();
    qDebug() << pUrl;
    qDebug() << pPath;

    QDateTime dateTime = QDateTime::currentDateTime();
    QSqlQuery addCacheEntry;
    addCacheEntry.prepare( QStringLiteral( "REPLACE INTO file_cache (url,path,ts)"
                                           " VALUES (:url,:path,:ts)" ) );
    addCacheEntry.bindValue( QStringLiteral( ":url" ), pUrl );
    addCacheEntry.bindValue( QStringLiteral( ":path" ), pPath );
    addCacheEntry.bindValue( QStringLiteral( ":ts" ), dateTime.toTime_t() );

    if ( !addCacheEntry.exec() ) {
        qWarning() << addCacheEntry.lastError();
    }

    commit();
}

void PersistanceLayer::addCustomEmoji( const QString &pTag, const QString &pPath, const QString &pHtml, const QString &pCategory )
{
    addCustomEmoji( pTag, pPath, pHtml, pCategory, QString( "" ) );
}

void PersistanceLayer::addCustomEmoji( const QString &pTag, const QString &pPath, const QString &pHtml, const QString &pCategory, const QString &pUnicode )
{
    addCustomEmoji( pTag, pPath, pHtml, pCategory, pUnicode, 0 );
}
void PersistanceLayer::addCustomEmoji( const QString &pTag, const QString &pPath, const QString &pHtml, const QString &pCategory, const QString &pUnicode, int pOrder )
{
    transaction();
    QSqlQuery addCustomEmojiQuery;
    addCustomEmojiQuery.prepare( QStringLiteral( "REPLACE INTO custom_emojis (id,file,html,unicode,category,sort_order)"
                                 " VALUES (:id,:file,:html,:unicode,:category,:sort_order)" ) );
    addCustomEmojiQuery.bindValue( QStringLiteral( ":id" ), pTag );
    addCustomEmojiQuery.bindValue( QStringLiteral( ":file" ), pPath );
    addCustomEmojiQuery.bindValue( QStringLiteral( ":html" ), pHtml );
    addCustomEmojiQuery.bindValue( QStringLiteral( ":unicode" ), pUnicode );
    addCustomEmojiQuery.bindValue( QStringLiteral( ":category" ), pCategory );
    addCustomEmojiQuery.bindValue( QStringLiteral( ":sort_order" ), pOrder );

    if ( !addCustomEmojiQuery.exec() ) {
        qWarning() << addCustomEmojiQuery.lastError();
    }

    commit();
}

void PersistanceLayer::addUserToBlockList( const QString &pUserId, const QString &pUsername )
{
    transaction();
    QSqlQuery addUserQuery;
    addUserQuery.prepare( QStringLiteral( "INSERT INO blocked_users(id, author) VALUES(:id,:author)" ) );
    addUserQuery.bindValue( ":id", pUserId );
    addUserQuery.bindValue( ":author", pUsername );

    if ( !addUserQuery.exec() ) {
        qWarning() << addUserQuery.lastError();
    }

    commit();

}


QHash<QString, QString> PersistanceLayer::getMessageByid( const QString &pId )
{
    QSqlQuery message;
    message.prepare( QStringLiteral( "SELECT * FROM messages LEFT JOIN blocked_users ON (messages.user_id=blocked_users.user_id) WHERE blocked_users.user_id IS NULL AND id=:id" ) );
    message.bindValue( QStringLiteral( ":id" ), pId );
    QHash<QString, QString> row;

    if ( !message.exec() ) {
        qWarning() << message.lastError();
    } else {

        QSqlRecord rec = message.record();
        int idCol = rec.indexOf( QStringLiteral( "id" ) );
        int ridCol = rec.indexOf( QStringLiteral( "rid" ) );
        int authorCol = rec.indexOf( QStringLiteral( "author" ) );
        int tsCol = rec.indexOf( QStringLiteral( "ts" ) );
        int messageCol = rec.indexOf( QStringLiteral( "message" ) );
        int typeCol = rec.indexOf( QStringLiteral( "type" ) );

        while ( message.next() ) {
            row[QStringLiteral( "id" )] = message.value( idCol ).toString();
            row[QStringLiteral( "rid" )] = message.value( ridCol ).toString();
            row[QStringLiteral( "author" )] = message.value( authorCol ).toString();
            row[QStringLiteral( "message" )] = message.value( messageCol ).toString();
            row[QStringLiteral( "ts" )] = message.value( tsCol ).toString();
            row[QStringLiteral( "type" )] = message.value( typeCol ).toString();
        }
    }

    return row;
}


QString PersistanceLayer::getUserName()
{
    QSqlQuery getName;
    getName.prepare( QStringLiteral( "SELECT username FROM user_data WHERE id=0" ) );

    QString name = "";

    if ( !getName.exec() ) {
        qWarning() << getName.lastError();
    } else {

        QSqlRecord rec = getName.record();
        int nameCol = rec.indexOf( QStringLiteral( "username" ) );

        while ( getName.next() ) {
            name = getName.value( nameCol ).toString();
        }
    }

    return name;
}

std::tuple<QString, QString> PersistanceLayer::getCurrentChannel()
{
    QSqlQuery getChannel;
    getChannel.prepare( QStringLiteral( "SELECT * FROM current_room WHERE id=0" ) );
    QString id;
    QString name;

    if ( !getChannel.exec() ) {
        qWarning() << getChannel.lastError();
    } else {
        while ( getChannel.next() ) {
            id = getChannel.value( QStringLiteral( "rid" ) ).toString();
            name = getChannel.value( QStringLiteral( "name" ) ).toString();
        }
    }

    return std::tuple<QString, QString>( id, name );
}

QString PersistanceLayer::getPassword()
{
    QSqlQuery getPass;
    getPass.prepare( QStringLiteral( "SELECT password FROM user_data WHERE id=0" ) );
    QString pass = "";

    if ( !getPass.exec() ) {
        qWarning() << getPass.lastError();
    } else {

        QSqlRecord rec = getPass.record();
        int nameCol = rec.indexOf( QStringLiteral( "password" ) );

        while ( getPass.next() ) {
            pass = getPass.value( nameCol ).toString();
        }
    }

    return pass;
}

QPair<QString, uint> PersistanceLayer::getToken()
{
    QSqlQuery getToken;
    getToken.prepare( QStringLiteral( "SELECT token, token_expire FROM user_data WHERE id=0" ) );
    // getToken.bindValue( ":id", 0 );
    QPair<QString, uint> token;

    if ( !getToken.exec() ) {
        qWarning() << getToken.lastError();
    } else {
        QSqlRecord rec = getToken.record();
        int tokenCol = rec.indexOf( QStringLiteral( "token" ) );
        int expireCol = rec.indexOf( QStringLiteral( "token_expire" ) );

        while ( getToken.next() ) {
            token.first = getToken.value( tokenCol ).toString();
            token.second = getToken.value( expireCol ).toUInt();
        }
    }

    return token;
}

QString PersistanceLayer::getUserId()
{
    QSqlQuery getUserId;
    getUserId.prepare( "SELECT id,user_id FROM user_data WHERE id=0" );
    //getUserId.addBindValue( ":id", 0 );
    QString userId;

    if ( !getUserId.exec() ) {
        qWarning() << "userid error" << getUserId.lastError();
    } else {
        QSqlRecord rec = getUserId.record();
        int userIdCol = rec.indexOf( QStringLiteral( "user_id" ) );

        while ( getUserId.next() ) {
            userId = getUserId.value( userIdCol ).toString();
        }
    }

    return userId;
}

QList<QJsonObject> PersistanceLayer::getMessagesByRid( const QString &pRid )
{
    QSqlQuery messages;
    messages.prepare( QStringLiteral( "SELECT * FROM messages WHERE rid=:rid ORDER BY ts DESC LIMIT 20" ) );
    messages.bindValue( QStringLiteral( ":rid" ), pRid );

    QList<QJsonObject> rows;
    rows.reserve( 50 );

    if ( !messages.exec() ) {
        qWarning() << QStringLiteral( "request room with rid: " ) << pRid;
        qWarning() << messages.lastError();
    } else {

        QSqlRecord rec = messages.record();
        int idCol = rec.indexOf( QStringLiteral( "id" ) );
        int jsonCol = rec.indexOf( QStringLiteral( "json" ) );
        int readCol = rec.indexOf( QStringLiteral( "read" ) );


        while ( messages.next() ) {
            //QHash<QString,QJsonObject> entry;
            QString id = messages.value( idCol ).toString();
            QString json = messages.value( jsonCol ).toString();
            bool read =  messages.value( readCol ).toBool();
            QJsonDocument doc = QJsonDocument::fromJson( json.toUtf8() );
            QJsonObject object = doc.object();
            object[QStringLiteral( "read" )] = read;
            //entry[id] = object;
            //   rows[id] = object;
            rows.append( object );
            //rows.append(entry);
        }
    }

    return rows;
}

QList<QJsonObject> PersistanceLayer::getMessagesByRid( const QString &pRid, qint64 pFrom, qint64 pTo )
{
    QSqlQuery messages;
    messages.prepare( QStringLiteral( "SELECT * FROM messages WHERE rid=:rid AND ts<=:from AND ts>=:to ORDER BY ts DESC LIMIT 50" ) );
    messages.bindValue( QStringLiteral( ":rid" ), pRid );
    messages.bindValue( QStringLiteral( ":from" ), ( qint64 )( pFrom / 1000 ) );
    messages.bindValue( QStringLiteral( ":to" ), ( qint64 )( pTo / 1000 ) );

    QList<QJsonObject> rows;
    rows.reserve( 50 );

    if ( !messages.exec() ) {
        qWarning() << "request room with rid: " << pRid;
        qWarning() << messages.lastError();
    } else {

        QSqlRecord rec = messages.record();
        int idCol = rec.indexOf( QStringLiteral( "id" ) );
        int jsonCol = rec.indexOf( QStringLiteral( "json" ) );
        int readCol = rec.indexOf( QStringLiteral( "read" ) );


        while ( messages.next() ) {
            QHash<QString, QJsonObject> entry;
            QString id = messages.value( idCol ).toString();
            QString json = messages.value( jsonCol ).toString();
            bool read =  messages.value( readCol ).toBool();
            QJsonDocument doc = QJsonDocument::fromJson( json.toUtf8() );
            QJsonObject object = doc.object();
            object[QStringLiteral( "read" )] = read;
            //entry["id"] = id;
            // entry["json"] = object;
            // rows.append(entry);
            rows.append( object );
        }
    }

    return rows;
}

QList<QJsonObject > PersistanceLayer::getMessagesByRid( const QString &pRid, qint64 pFrom, int pLimit )
{
    QSqlQuery messages;
    messages.prepare( QStringLiteral( "SELECT * FROM messages WHERE rid=:rid AND ts<=:from ORDER BY ts DESC LIMIT :limit" ) );
    messages.bindValue( QStringLiteral( ":rid" ), pRid );
    messages.bindValue( QStringLiteral( ":from" ), ( qint64 )( pFrom ) );
    messages.bindValue( QStringLiteral( ":limit" ), pLimit );

    QList<QJsonObject > rows;
    rows.reserve( 50 );

    if ( !messages.exec() ) {
        qWarning() << "request room with rid: " << pRid;
        qWarning() << messages.lastError();
    } else {

        QSqlRecord rec = messages.record();
        int idCol = rec.indexOf( QStringLiteral( "id" ) );
        int jsonCol = rec.indexOf( QStringLiteral( "json" ) );
        int readCol = rec.indexOf( QStringLiteral( "read" ) );


        while ( messages.next() ) {
            // QHash<QString,QJsonObject> entry;
            QString id = messages.value( idCol ).toString();
            QString json = messages.value( jsonCol ).toString();
            bool read =  messages.value( readCol ).toBool();
            QJsonDocument doc = QJsonDocument::fromJson( json.toUtf8() );
            QJsonObject object = doc.object();

            if ( !object.size() ) {
                qWarning() << "invalid message Json: " << json;
            } else {
                object[QStringLiteral( "read" )] = read;

                //   QJsonValue idJson = id;
                //   entry["json"] = object;
                rows.append( object );
            }
        }
    }

    return rows;
}

QList<QPair<QString, QString >> PersistanceLayer::getListOfBlockedUsers()
{
    QSqlQuery usersQuery;
    usersQuery.prepare( QStringLiteral( "SELECT * FROM blocked_users" ) );
    QList<QPair<QString, QString >> retList;

    if ( !usersQuery.exec() ) {
        qWarning() << "getList of blocked users ";
        qWarning() << usersQuery.lastError();
    } else {
        QSqlRecord rec = usersQuery.record();
        int idCol = rec.indexOf( "id" );
        int nameCol = rec.indexOf( "author" );

        while ( usersQuery.next() ) {
            QPair<QString, QString> pair;
            pair.first = usersQuery.value( idCol ).String;
            pair.second = usersQuery.value( nameCol ).String;
            retList.append( pair );
        }
    }

    return retList;
}


QList<QVariantHash> PersistanceLayer::getChannels( void )
{
    QSqlQuery rooms;
    rooms.prepare( QStringLiteral( "SELECT * FROM rooms LEFT JOIN (SELECT DISTINCT rid FROM messages ORDER BY ts DESC) AS sub ON rooms.id = sub.rid " ) );
    QList<QVariantHash> roomsList;
    roomsList.reserve( 100 );

    if ( !rooms.exec() ) {
        qWarning() << rooms.lastError();
    } else {
        QSqlRecord rec = rooms.record();
        int idCol = rec.indexOf( QStringLiteral( "id" ) );
        int nameCol = rec.indexOf( QStringLiteral( "name" ) );
        int joinedCol = rec.indexOf( QStringLiteral( "joined" ) );
        int typeCol = rec.indexOf( QStringLiteral( "type" ) );
        int readOnlyCol = rec.indexOf( QStringLiteral( "read_only" ) );
        int mutedCol =  rec.indexOf( QStringLiteral( "muted" ) );
        int archivedCol = rec.indexOf( QStringLiteral( "archived" ) );
        int blocked = rec.indexOf( QStringLiteral( "blocked" ) );
        int usernameCol = rec.indexOf( QStringLiteral( "username" ) );

        while ( rooms.next() ) {
            QVariantHash roomsHash;
            QString muted = rooms.value( mutedCol ).toString();
            roomsHash[QStringLiteral( "id" )] = rooms.value( idCol ).toString();
            roomsHash[QStringLiteral( "name" )] = rooms.value( nameCol ).toString();
            roomsHash[QStringLiteral( "joined" )] = rooms.value( joinedCol ).toBool();
            roomsHash[QStringLiteral( "type" )] = rooms.value( typeCol ).toString();
            roomsHash[QStringLiteral( "readOnly" )] = rooms.value( readOnlyCol ).toBool();
            roomsHash[QStringLiteral( "muted" )] = muted;
            roomsHash[QStringLiteral( "list" )] = muted.split( ',' );
            roomsHash[QStringLiteral( "archived" )] = rooms.value( archivedCol ).toBool();
            roomsHash[QStringLiteral( "blocked" )] = rooms.value( blocked ).toBool();
            roomsHash[QStringLiteral( "username" )] = rooms.value( usernameCol ).toString();
            roomsList.append( roomsHash );
        }
    }

    return roomsList;
}

QList< QHash<QString, QString > > PersistanceLayer::getCustomEmojis()
{
    QSqlQuery emojiQuery;
    emojiQuery.prepare( QStringLiteral( "SELECT id,file,html,category,unicode,sort_order FROM custom_emojis" ) );
    QList<QHash<QString, QString> > returnList;
    returnList.reserve( 3000 );

    if ( !emojiQuery.exec() ) {
        qWarning() << emojiQuery.lastError();
    } else {

        QSqlRecord rec = emojiQuery.record();

        int idCol = rec.indexOf( QStringLiteral( "id" ) );
        int fileCol = rec.indexOf( QStringLiteral( "file" ) );
        int htmlCol = rec.indexOf( QStringLiteral( "html" ) );
        int catCol = rec.indexOf( QStringLiteral( "category" ) );
        int unicodeCol = rec.indexOf( QStringLiteral( "unicode" ) );
        int orderCol = rec.indexOf( QStringLiteral( "sort_order" ) );

        while ( emojiQuery.next() ) {
            QHash<QString, QString> entry;
            entry[QStringLiteral( "id" )] = emojiQuery.value( idCol ).toString();
            entry[QStringLiteral( "file" )] = emojiQuery.value( fileCol ).toString();
            entry[QStringLiteral( "html" )] = emojiQuery.value( htmlCol ).toString();
            entry[QStringLiteral( "category" )] = emojiQuery.value( catCol ).toString();
            entry[QStringLiteral( "unicode" )] = emojiQuery.value( unicodeCol ).toString();
            entry[QStringLiteral( "sort_order" )] = emojiQuery.value( orderCol ).toString();

            returnList.append( entry );
        }
    }

    return returnList;
}

QString PersistanceLayer::getFileCacheEntry( const QString &pUrl )
{
    QSqlQuery cacheEntry;
    cacheEntry.prepare( QStringLiteral( "SELECT path FROM file_cache WHERE url=:url" ) );
    cacheEntry.bindValue( QStringLiteral( ":url" ), pUrl );
    QString returnString = "";

    if ( !cacheEntry.exec() ) {
        qWarning() << cacheEntry.lastError();
    } else {

        QSqlRecord rec = cacheEntry.record();
        int pathCol = rec.indexOf( QStringLiteral( "path" ) );

        while ( cacheEntry.next() ) {
            returnString = cacheEntry.value( pathCol ).toString();
        }

        if ( returnString != "" ) {
            QFile cachedFile( returnString );

            if ( !cachedFile.exists() ) {
                removeFileCacheEntry( pUrl, returnString );
                returnString = "";
            }
        }
    }

    return returnString;
}

QString PersistanceLayer::getSetting( const QString &pProperty )
{
    QSqlQuery settingsEntry;
    settingsEntry.prepare( QStringLiteral( "SELECT property,value FROM app_settings WHERE property=:property" ) );
    settingsEntry.bindValue( QStringLiteral( ":property" ), pProperty );
    QString returnString = "";

    if ( !settingsEntry.exec() ) {
        qWarning() << settingsEntry.lastError();
    } else {
        QSqlRecord rec = settingsEntry.record();

        int valueClol = rec.indexOf( QStringLiteral( "value" ) );

        while ( settingsEntry.next() ) {
            returnString = settingsEntry.value( valueClol ).toString();
        }
    }

    return returnString;
}


void PersistanceLayer::removeFileCacheEntry( const QString &pUrl )
{
    QString fileName = getFileCacheEntry( pUrl );
    removeFileCacheEntry( pUrl, fileName );
}

void PersistanceLayer::removeFileCacheEntry( const QString &pUrl, const QString &pPath )
{
    transaction();

    if ( pUrl != "" && pPath != "" ) {
        QFile cachedFile( pPath );

        if ( cachedFile.exists() ) {
            if ( !cachedFile.remove() ) {
                qWarning() << cachedFile.errorString();
            }
        }
    }

    QSqlQuery cacheEntry;
    cacheEntry.prepare( QStringLiteral( "DELETE FROM file_cache WHERE url=:url" ) );
    cacheEntry.bindValue( QStringLiteral( ":url" ), pUrl );

    if ( !cacheEntry.exec() ) {
        qWarning() << cacheEntry.lastError();
    }

    commit();
}


QString PersistanceLayer::getNewVideoPath( void )
{
    QString path;
    path = QStandardPaths::writableLocation( QStandardPaths::MoviesLocation ) + QStringLiteral( "/recordings" );

    if ( path != "" ) {
        QDir dir( path );

        if ( !dir.exists() ) {
            if ( !dir.mkpath( path ) ) {
                qWarning() << "failed to create folder: " << path;
            }
        }

        QDateTime dateTime = QDateTime::currentDateTime();
        uint timeStamp = dateTime.toTime_t();
        QFile file( path + '/' + timeStamp + QStringLiteral( ".mp4" ) );

        if ( !file.exists() ) {
            //file.open( QIODevice::ReadWrite );
            //file.close();
        }

        qDebug() << path + '/' + QString::number( timeStamp ) + ".mp4";
        return path + '/' + QString::number( timeStamp ) + QStringLiteral( ".mp4" );
    } else {
        return "";
    }
}

QString PersistanceLayer::getNewImagePath( void )
{
    QString path;
    path = QStandardPaths::writableLocation( QStandardPaths::PicturesLocation ) + QStringLiteral( "/recordings" );

    if ( path != "" ) {
        QDir dir( path );

        if ( !dir.exists() ) {
            if ( !dir.mkpath( path ) ) {
                qWarning() << "failed to create folder: " << path;
            }
        }

        QDateTime dateTime = QDateTime::currentDateTime();
        uint timeStamp = dateTime.toTime_t();
        QFile file( path + '/' + timeStamp + QStringLiteral( ".jpg" ) );

        if ( !file.exists() ) {
            //file.open( QIODevice::ReadWrite );
            //file.close();
        }

        qDebug() << path + '/' + QString::number( timeStamp ) + QStringLiteral( ".jpg" );
        return path + '/' + QString::number( timeStamp ) + QStringLiteral( ".jpg" );
    } else {
        return "";
    }
}



PersistanceLayer::~PersistanceLayer()
{
    mDb.close();
}

void PersistanceLayer::init()
{
    QString directory = QStandardPaths::writableLocation( QStandardPaths::AppLocalDataLocation );
    QString filePath = directory + QStringLiteral( "/data.sqlite" );
    qDebug() << "SQLITE: " << filePath;

    mDb.setDatabaseName( filePath );
    QDir dbPath( directory );

    if ( !dbPath.exists() ) {
        dbPath.mkpath( directory );
    }

    QFile file( filePath );

    if ( !file.exists() ) {
        file.open( QIODevice::ReadWrite );
        file.close();
    }

    if ( !mDb.open() ) {
        qDebug() << mDb.lastError().text() + "file used:" + filePath;
        throw std::runtime_error( mDb.lastError().text().toStdString() + "file used:" + filePath.toStdString() );
    }

    initShema();
    emit( ready() );
}
