/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/


#include "emoji.h"

Emoji::Emoji( QString name, QString extension, QString category ): mCategory( category )
{

    this->mIdentifier = ':' + name + ':';
    mName = name;
    mExtension = extension;
    this->mType = QStringLiteral( "emoji" );
}

Emoji::Emoji( QString name, QString category, QString filePath, QString html ): mCategory( category )
{
    this->mIdentifier = name;
    QFileInfo fileInfo( filePath );
    mFilePath = filePath;
    mhtml = html;
    mExtension = fileInfo.completeSuffix();
}

Emoji::Emoji( QString name, QString category, QString filePath, QString html, QString unicode ): mCategory( category ), mUnicodeChar( unicode )
{
    this->mIdentifier = name;
    QFileInfo fileInfo( filePath );
    mFilePath = filePath;
    mhtml = html;
    mExtension = fileInfo.completeSuffix();
}

Emoji::Emoji( QString name, QString category, QString filePath, QString html, QString unicode, int order ): mCategory( category ), mUnicodeChar( unicode ), mOrder( order )
{
    this->mIdentifier = name;
    QFileInfo fileInfo( filePath );
    mFilePath = filePath;
    mhtml = html;
    mExtension = fileInfo.completeSuffix();
}

QString Emoji::getIdentifier() const
{
    return mIdentifier;
}

QVariantMap Emoji::toQVariantMap()
{
    QVariantMap entry;
    entry[QStringLiteral( "category" )] = mCategory;
    entry[QStringLiteral( "text" )] = mIdentifier;
    entry[QStringLiteral( "file" )] = mFilePath;
    entry[QStringLiteral( "order" )] = mOrder;
    return entry;
}

QString Emoji::getHtml() const
{
    return mhtml;
}

void Emoji::setHtml( const QString &value )
{
    mhtml = value;
}

QString Emoji::getExtension() const
{
    return mExtension;
}

QString Emoji::getName() const
{
    return mName;
}

void Emoji::setName( const QString &name )
{
    mName = name;
}

QString Emoji::getUnicodeChar() const
{
    return mUnicodeChar;
}

void Emoji::setUnicodeChar( const QString &unicodeChar )
{
    mUnicodeChar = unicodeChar;
}

int Emoji::getOrder() const
{
    return mOrder;
}

void Emoji::setOrder( int pOrder )
{
    mOrder = pOrder;
}



QString Emoji::getCategory() const
{
    return mCategory;
}

void Emoji::setCategory( const QString &category )
{
    mCategory = category;
}
